// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file RosenbrockStationary.h */

#ifndef AMDIS_ROSENBROCKSTATIONARY_H
#define AMDIS_ROSENBROCKSTATIONARY_H

#include "AMDiS_fwd.h"
#include "ProblemVec.h"
#include "SystemVector.h"
#include "time/RosenbrockMethod.h"

namespace AMDiS {

  class RosenbrockStationary : public ProblemVec
  {
  public:
    RosenbrockStationary(std::string name)
      : ProblemVec(name),
	first(true)
    {}

    DOFVector<double>* getStageSolution(int i)
    {
      return stageSolution->getDOFVector(i);
    }

    DOFVector<double>* getUnVec(int i)
    {
      return unVec->getDOFVector(i);
    }

    DOFVector<double>* getPhiSum()
    {
      return phiSum;
    }

    void setTauPtr(double *ptr)
    {
      tauPtr = ptr;
    }

    void acceptTimestep();

    void buildAfterCoarsen(AdaptInfo *adaptInfo, Flag flag,
			   bool asmMatrix, bool asmVector);

    void solve(AdaptInfo *adaptInfo, bool fixedMatrix);

    void setRosenbrockMethod(RosenbrockMethod *method)
    {
      rm = method;
      init();
    }

  protected:
    void init();

  protected:
    RosenbrockMethod *rm;

    SystemVector *stageSolution, *unVec, *newUn, *tmp, *lowSol;

    std::vector<SystemVector*> stageSolutions;

    DOFVector<double> *phiSum, *tmpDof;

    double *tauPtr;

    bool first;
  };

}

#endif // AMDIS_ROSENBROCKSTATIONARY_H
